@@ -21,10 +21,10 @@
 
 #include <linux/elf.h>
 #include <linux/kernel.h>
-#include <linux/mm.h>
 #ifndef TESTING
 #  include <linux/string.h>
 #endif
+#include <linux/version.h>
 
 #include <asm/console.h>
 #include <asm/hwrpb.h>
@@ -38,11 +38,14 @@
 #include <setjmp.h>
 #include <utils.h>
 
-#define PASTE(x)		#x
-#define VERSION(maj,min)	PASTE(maj) "." PASTE(min)
-
 #ifndef elf_check_arch
-# define elf_check_arch(e)	1
+# define aboot_elf_check_arch(e)	1
+#else
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#  define aboot_elf_check_arch(e)        elf_check_arch(e)
+# else
+#  define aboot_elf_check_arch(e)        elf_check_arch(e->e_machine)
+# endif
 #endif
 
 struct bootfs *	bfs = 0;		/* filesystem to boot from */
@@ -95,7 +98,7 @@ first_block (const char *buf, long blocksize)
 			printf("aboot: not an executable ELF file\n");
 			return -1;
 		}
-		if (!elf_check_arch(elf->e_machine)) {
+		if (!aboot_elf_check_arch(elf)) {
 			printf("aboot: ELF executable not for this machine\n");
 			return -1;
 		}
@@ -110,15 +113,26 @@ first_block (const char *buf, long blocksize)
 		nchunks = elf->e_phnum;
 		start_addr = phdrs[0].p_vaddr; /* assume they are sorted */
 		entry_addr = elf->e_entry;
+#ifdef DEBUG
+		printf("aboot: %d program headers, start address %#lx, entry %#lx\n",
+		       nchunks, start_addr, entry_addr);
+#endif
 		for (i = 0; i < elf->e_phnum; ++i) {
 			chunks[i].addr   = phdrs[i].p_vaddr;
 			chunks[i].offset = phdrs[i].p_offset;
 			chunks[i].size   = phdrs[i].p_filesz;
+#ifdef DEBUG
+			printf("aboot: segment %d vaddr %#lx offset %#lx size %#lx\n",
+			       i, chunks[i].addr, chunks[i].offset, chunks[i].size);
+#endif
 		}
 		bss_start = (char *) (phdrs[elf->e_phnum - 1].p_vaddr +
 				      phdrs[elf->e_phnum - 1].p_filesz);
 		bss_size = (phdrs[elf->e_phnum - 1].p_memsz -
 			    phdrs[elf->e_phnum - 1].p_filesz);
+#ifdef DEBUG
+		printf("aboot: bss at 0x%p, size %#lx\n", bss_start, bss_size);
+#endif
 	} else {
 		/* Fail silently, it might be a compressed file */
 		return -1;
@@ -203,8 +217,7 @@ main_ (void)
 
 	cons_init();
 
-	printf("aboot: Linux/Alpha SRM bootloader version "
-	       VERSION(ABOOT_MAJOR, ABOOT_MINOR) "\n");
+	printf("aboot: Linux/Alpha SRM bootloader version "ABOOT_VERSION"\n");
 
 	/* don't know how to deal with this yet */
 	if (INIT_HWRPB->pagesize != 8192) {
