@@ -55,25 +55,41 @@ static struct bootfs *bootfs[] = {
 };
 
 /*
- * Attempt a raw boot (header-less kernel follows right after aboot).
+ * Attempt a "raw" boot (uncompressed ELF kernel follows right after aboot).
+ *
+ * This will eventually be rewritten to accept compressed kernels
+ * (along with net_aboot).  It should also be merged with the other
+ * load methods as there is some code duplication here we don't want.
  */
 
-/* ??? Assume the kernel is no bigger than 10MB.  */
-#define KERNEL_SIZE (10*1024*1024)
-
 int
 load_raw (long dev)
 {
 	extern char _end;
+	char *buf;
 	long aboot_size = &_end - (char *) BOOT_ADDR;
 	long ksect = (aboot_size + SECT_SIZE - 1) / SECT_SIZE + BOOT_SECTOR;
+	long nread;
 	int i;
 
-	printf("aboot: loading raw kernel...\n");
+	printf("aboot: loading kernel from boot sectors...\n");
+
+	/* We only need the program headers so this should be fine */
+	buf = malloc(SECT_SIZE);
+
+	/* Read ELF headers: */
+	nread = cons_read(dev, buf, SECT_SIZE, ksect * SECT_SIZE);
+	if (nread != SECT_SIZE) {
+		printf("aboot: read returned %ld instead of %ld bytes\n",
+		       nread, (long) SECT_SIZE);
+		return -1;
+	}
+	if (first_block(buf, SECT_SIZE) < 0) {
+		return -1;
+	}
 
 	for (i = 0; i < nchunks; ++i) {
 		char *dest;
-		long nread;
 
 		printf("aboot: segment %d, %ld bytes at %#lx\n", i, chunks[i].size,
 		       chunks[i].addr);
@@ -83,14 +99,15 @@ load_raw (long dev)
 		dest = (char *) chunks[i].addr;
 #endif		      
 
-		nread = cons_read(dev, ksect * SECT_SIZE + chunks[i].offset,
-				  chunks[i].size, dest);
+		nread = cons_read(dev, dest, chunks[i].size,
+				  chunks[i].offset + ksect * SECT_SIZE);
 		if (nread != chunks[i].size) {
 			printf("aboot: read returned %ld instead of %ld bytes\n",
 			       nread, chunks[i].size);
 			return -1;
 		}
 	}
+	return 0;
 }
 
 
@@ -103,8 +120,7 @@ load_uncompressed (int fd)
 
 	buf = malloc(bfs->blocksize);
 
-	/* read a.out header: */
-
+	/* read ELF headers: */
 	nread = (*bfs->bread)(fd, 0, 1, buf);
 	if (nread != bfs->blocksize) {
 		printf("aboot: read returned %ld instead of %ld bytes\n",
@@ -503,13 +519,13 @@ print_help(void)
 }
 
 static void
-get_options (long dev)
+get_aboot_options (long dev)
 {
 	int preset      = 0; /* no preset */
 	int interactive = 0;  /* non-interactive */
 
 #ifdef DEBUG
-	printf("get_options(%lx)\n",dev);
+	printf("get_aboot_options(%lx)\n",dev);
 #endif
 
 	/* Forms of -flags argument from SRM */
@@ -569,8 +585,7 @@ get_options (long dev)
 			/* Otherwise, clear out kernel_args and prompt the user */
 			kernel_args[0] = 0;
 			if (first) {
-				printf("Welcome to aboot %d.%d\n",
-				       ABOOT_MAJOR, ABOOT_MINOR);
+				printf("Welcome to aboot " ABOOT_VERSION "\n");
 				print_help();
 				first = 0;
 			}
@@ -720,19 +735,22 @@ load (long dev)
 	fname = boot_file;
 	if (fname[0] == '-' && fname[1] == '\0') {
 		/* a single "-" implies raw boot: */
-		return load_raw(dev);
-	}
-	/* if there's no disklabel, boot_part will be ignored anyway */
-	bfs = mount_fs(dev, boot_part);
-	if (!bfs) {
-		printf("aboot: mount of partition %d failed\n", boot_part);
-		return -1;
-	}
-	if (read_kernel(fname) < 0) {
-		return -1;
+		if (load_raw(dev) < 0) {
+			return -1;
+		}
+	} else {
+		/* if there's no disklabel, boot_part will be ignored anyway */
+		bfs = mount_fs(dev, boot_part);
+		if (!bfs) {
+			printf("aboot: mount of partition %d failed\n", boot_part);
+			return -1;
+		}
+		if (read_kernel(fname) < 0) {
+			return -1;
+		}
 	}
 	/* clear bss: */
-	printf("aboot: zero-filling %ld bytes at %p\n", bss_size, bss_start);
+	printf("aboot: zero-filling %ld bytes at 0x%p\n", bss_size, bss_start);
 #ifndef TESTING
 	memset((char*)bss_start, 0, bss_size);
 #endif
@@ -787,7 +805,7 @@ load_kernel (void)
 	get_disklabel(dev);
 
 	while (1) {
-		get_options(dev);
+		get_aboot_options(dev);
 		result = load(dev);
 		if (result != -1)
 			break;
