@@ -6,16 +6,30 @@
  * This file has been ported from the DEC 32-bit Linux version
  * by David Mosberger (davidm@cs.arizona.edu).
  */
-#include <linux/stat.h>
-#include <linux/types.h>
-
-#undef __KERNEL__
-#include <linux/ext2_fs.h>
-#define __KERNEL__
-
-#include <linux/fs.h>
 #ifndef TESTING
 #  include <linux/string.h>
+#endif
+#include <linux/stat.h>
+#include <linux/types.h>
+#include <linux/version.h>
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
+
+#  undef __KERNEL__
+#  include <linux/ext2_fs.h>
+#  define __KERNEL__
+#  include <linux/fs.h>
+#  ifndef TESTING
+#    include <linux/string.h>
+#  endif
+
+#else /* Linux 2.4.0 or later */
+
+#  undef __KERNEL__
+#  include <linux/ext2_fs.h>
+#  include <linux/fs.h>
+#  define __KERNEL__
+
 #endif
 
 #include "bootfs.h"
@@ -152,7 +166,7 @@ static struct ext2_inode *ext2_iget(int ino)
 	printf("group is %d\n", group);
 #endif
 	offset = partition_offset
-		+ ((long) gds[group].bg_inode_table * ext2fs.blocksize)
+		+ ((long) gds[group].bg_inode_table * (long)ext2fs.blocksize)
 		+ (((ino - 1) % EXT2_INODES_PER_GROUP(&sb))
 		   * EXT2_INODE_SIZE(&sb));
 #ifdef DEBUG
@@ -216,7 +230,7 @@ static int ext2_blkno(struct ext2_inode *ip, int blkoff)
 	int blkno;
 	int iblkno;
 	int diblkno;
-	long offset;
+	unsigned long offset;
 
 	ilp = (unsigned int *)iblkbuf;
 	dlp = (unsigned int *)diblkbuf;
@@ -237,7 +251,7 @@ static int ext2_blkno(struct ext2_inode *ip, int blkoff)
 
 		/* Read the indirect block */
 		if (cached_iblkno != iblkno) {
-			offset = partition_offset + iblkno * ext2fs.blocksize;
+			offset = partition_offset + (long)iblkno * (long)ext2fs.blocksize;
 			if (cons_read(dev, iblkbuf, ext2fs.blocksize, offset)
 			    != ext2fs.blocksize)
 			{
@@ -262,7 +276,7 @@ static int ext2_blkno(struct ext2_inode *ip, int blkoff)
 
 		/* Read in the double-indirect block */
 		if (cached_diblkno != diblkno) {
-			offset = partition_offset + diblkno * ext2fs.blocksize;
+			offset = partition_offset + (long) diblkno * (long) ext2fs.blocksize;
 			if (cons_read(dev, diblkbuf, ext2fs.blocksize, offset)
 			    != ext2fs.blocksize)
 			{
@@ -282,7 +296,7 @@ static int ext2_blkno(struct ext2_inode *ip, int blkoff)
 		/* Read the indirect block */
     
 		if (cached_iblkno != iblkno) {
-			offset = partition_offset + iblkno * ext2fs.blocksize;
+			offset = partition_offset + (long) iblkno * (long) ext2fs.blocksize;
 			if (cons_read(dev, iblkbuf, ext2fs.blocksize, offset)
 			    != ext2fs.blocksize)
 			{
@@ -332,7 +346,7 @@ static int ext2_breadi(struct ext2_inode *ip, long blkno, long nblks,
 			memset(buffer, 0, nbytes);
 		} else {
 			/* Read it for real */
-			offset = partition_offset + dev_blkno*ext2fs.blocksize;
+			offset = partition_offset + (long) dev_blkno* (long) ext2fs.blocksize;
 #ifdef DEBUG
 			printf("ext2_bread: reading %ld bytes at offset %ld\n",
 			       nbytes, offset);
