@@ -3,6 +3,9 @@
 
 #include <asm/hwrpb.h>
 
+#ifdef TESTING
+#define pal_init()
+#else
 extern int		printf (const char *fmt, ...);
 extern struct pcb_struct *find_pa (unsigned long vptb, struct pcb_struct *pcb);
 extern void		pal_init (void);
@@ -10,5 +13,9 @@ extern void		pal_init (void);
 extern void *		malloc (size_t size);
 extern void		free (void *ptr);
 extern void		getline (char *buf, int maxlen);
+#endif
+
+extern unsigned long	memory_end(void);
+extern unsigned long	free_mem_ptr;
 
 #endif /* utils_h */
