@@ -291,9 +291,9 @@ int vsprintf(char *buf, const char *fmt, va_list args)
 			num = va_arg(args, unsigned long);
 		else if (qualifier == 'h')
 			if (flags & SIGN)
-				num = va_arg(args, short);
+				num = va_arg(args, int);
 			else
-				num = va_arg(args, unsigned short);
+				num = va_arg(args, unsigned int);
 		else if (flags & SIGN)
 			num = va_arg(args, int);
 		else
