@@ -9,6 +9,10 @@
 #include <linux/hdreg.h>
 #include "library.h"
 
+/* from linux/fs.h */
+#define BLKSSZGET  _IO(0x12,104)   /* get block device sector size */
+#define BLKGETSIZE _IO(0x12,96)    /* return device size */
+
 int label_modified=0;
 
 int
@@ -53,23 +57,62 @@ print_disklabel(int fd,struct disklabel *d)
 	}
 }
 
-/* hardcoded until I get the system totally up */
-
-int total_disk_size=530*1024*(1024/512);
+int total_disk_size, heads, sectors, cylinders;
+/* FIXME: use BLKSSZGET to get sector size. (depends on linux >=2.4) */
+int sector_size=512;
 
 #ifdef __linux__
+
 void
 set_disk_size (int fd)
 {
 	struct hd_geometry hd;
-	if(ioctl(fd,HDIO_GETGEO,&hd)) {
+	unsigned long blocks;
+	unsigned int sectors_per_block;
+	int r1, r2;
+	
+	sectors_per_block = sector_size / 512;
+	heads = sectors = cylinders = blocks = 0;
+
+	r1 = ioctl(fd,HDIO_GETGEO,&hd);
+
+	if (r1) {
 		perror("ioctl HDIO_GETGEO");
-		fprintf(stderr,"Assuming disk size is %d\n",total_disk_size);
 	} else {
-		total_disk_size=hd.heads*hd.sectors*hd.cylinders;
-		fprintf(stderr,"I think your disk is %dK total size\n",total_disk_size/2);
-		fprintf(stderr,"If I'm wrong, override the size with the 'size num' option\n\n");
+		heads = hd.heads;
+		sectors = hd.sectors;
+		cylinders = hd.cylinders;
+		if (heads * sectors * cylinders == 0) { r1 = -1; }
+		/* fdisk says: "never use hd.cylinders - it is truncated" 
+		   if BLKGETSIZE works we'll calculate our own value for
+		   cylinders in a little bit, but for now, use it anyway */
+		total_disk_size=(heads*sectors*cylinders)/2;
 	}
+	
+	r2 = ioctl(fd,BLKGETSIZE, &blocks);
+
+	if (r2) {
+		perror("ioctl BLKGETSIZE");
+	}
+
+	if (r1 && r2) {
+		if (!total_disk_size) {
+			fprintf(stderr, "Unable to get disk size. Please specify it with the size [size_in_sectors] option.\n\n");
+		}
+		return;
+	}
+
+	if (r1 == 0 && r2 == 0) {
+		total_disk_size = blocks / 2;
+		cylinders = blocks / (heads * sectors);
+		cylinders /= sectors_per_block;
+	} else if (r1 == 0) {
+		fprintf(stderr, "Unable to get disk geometry. Guessing number of sectors from disk size.\n");
+		cylinders = heads = 1;
+		sectors = blocks / sectors_per_block;
+	}
+	fprintf(stderr,"%d heads, %d sectors, %d cylinders %dK total size\n", 
+		heads, sectors, cylinders, total_disk_size);
 }
 #endif
 
@@ -122,7 +165,7 @@ main (int argc,char **argv)
 	struct disklabel d;
 	int fd,x;
 
-	if(argc<2) {
+	if(argc < 3) {
 		usage(argv[0]);
 		exit(1);
 	}
